package cds.charac.model;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import cds.charac.tools.*; 
import cds.characterisationlibrary.*;
import net.ivoa.xml.characterisation.characterisationV111.CharacterisationType;


/**
 * Cette classe offre des methodes qui permettent de generer
 *  un fichier(ou un repertoire des fichiers) Charac au format XML ou uty
 * 
 * @author Omar Benjelloun
 * 
 */
public class CharacFile {

	private File dfltXmlFolder = new File("defaultXml");
	private File dfltUtypeFolder = new File("defaultUtype");

	/**
	 * cette fonction permet de creer le fichier uty correspondant au fichier
	 * fits et au fichier de mapping donns en paramtre, cette fonction prend en
	 * charge les axes spcifique et generiques
	 * 
	 * @param utypeF
	 *            le fichier utype de retour de la fonction
	 * @param fitsF
	 *            le fichier fits
	 * @param mapF
	 *            le fichier de mapping
	 * @return le fichier utype cre
	 */

	public File createUtypeFile(File utypeF, File fitsF, File mapF) {

		MoCharacGenerator cg = new MoCharacGenerator(fitsF, mapF);
		if (cg.objectValide) {
			Vector<HashMap<String, String>> vSpecicAxis = cg.specificAxis();
			try {
				FileWriter fw = new FileWriter(utypeF);
				BufferedWriter bw = new BufferedWriter(fw);
				PrintWriter pw = new PrintWriter(bw);
				HashMap<String, String> utyVal;
				// ecriture de la charac. des axes specifiques
				// System.out.println("Axes Specifique :"+vSpecicAxis.size());
				// System.out.println("Axes Specifique
				// :"+vSpecicAxis.toString());
				for (int i = 0; i < vSpecicAxis.size(); i++) {
					utyVal = vSpecicAxis.elementAt(i);// utyVal=new
					// HashMap<String,
					// String>();
					Iterator<String> iter0 = utyVal.keySet().iterator();
					// Ecriture du AxisName comme 1er element de la
					// characterisation de l'axe
					while (iter0.hasNext()) {
						String uty = iter0.next();
						String utypeValue = (String) utyVal.get(uty);

						if (uty.trim().endsWith(".AxisName")) {
							pw.println("%CharacterisationAxis " + (i + 1));
							pw.println("%" + uty.trim() + " " + utypeValue);
							break;
						}
					}
					// Ecriture du reste de la characterisation de l'axe
					Iterator<String> iter1 = utyVal.keySet().iterator();
					while (iter1.hasNext()) {
						String uty = iter1.next();
						String utypeValue = (String) utyVal.get(uty);
						if (!uty.trim().endsWith(".AxisName"))
							pw.println("%" + uty.trim() + " " + utypeValue);
					}
					//System.out.println("Axes Specifique " + i + " " + utyVal.toString());

					pw.println();
				}
				Vector<HashMap<String, String>> vGenericAxis = cg.genericAxis();
				// ecriture de la charac. des axes generiques
				// System.out.println("Axes Generiqueique
				// :"+vGenericAxis.toString());
				for (int j = 0; j < vGenericAxis.size(); j++) {
					utyVal = vGenericAxis.elementAt(j);// utyVal=new
					// HashMap<String,
					// String>();
					Iterator<String> iter0 = utyVal.keySet().iterator();
					// Ecriture du AxisName comme 1er element de la
					// characterisation de l'axe
					while (iter0.hasNext()) {
						String uty = iter0.next();
						String utypeValue = (String) utyVal.get(uty);

						if (uty.trim().endsWith(".AxisName")) {
							pw.println("%CharacterisationAxis "
									+ (j + vSpecicAxis.size() + 1));
							pw.println("%" + uty.trim() + " " + utypeValue);
							break;
						}

					}
					// Ecriture du du reste de la characterisation de l'axe
					Iterator<String> iter1 = utyVal.keySet().iterator();
					while (iter1.hasNext()) {
						String uty = iter1.next();
						String utypeValue = (String) utyVal.get(uty);
						if (!uty.trim().endsWith(".AxisName"))
							pw.println("%" + uty.trim() + " " + utypeValue);
					}
					pw.println();
				}
				pw.close();
			} catch (FileNotFoundException e) {
				System.out.println(e.getMessage());

			} catch (IOException e) {
				System.out.println(e.getMessage());
			}
		}
		return utypeF;
	}

	/**
	 * cette fonction surcharge la fonction createUtypeFile(File utypeF,File
	 * fitsF,File mapF) le fichier utype ainsi gnr aura un nom gnrer
	 * automatiquement  partir du fichier FITS exemple :
	 * nomFits.fits==>nomFits.uty
	 * 
	 * @param fitsF
	 *            fichier fits
	 * @param mapF
	 *            fichier de mapping
	 * @return fichier utype cree
	 */
	public File createUtypeFile(File fitsF, File mapF) {
		File utypeF = null;
		// construction du nom de fichier utype
		String parfileName = fitsF.getName();
		parfileName = MappingTools.ChangeFileExtension(parfileName, "uty");
		

		if (!(dfltUtypeFolder.exists() && dfltUtypeFolder.isDirectory())) {
			dfltUtypeFolder.mkdir();
			System.out.println("Creation du rpertoire: ." + File.separator
					+ dfltUtypeFolder.getName());
		}

		String rep = dfltUtypeFolder.getName();
		utypeF = new File(rep + File.separator + parfileName);

		this.createUtypeFile(utypeF, fitsF, mapF);

		return utypeF;

	}

	/**
	 * Cette fonction cherche dans un repertoire donn en parametre tous les
	 * fichiers fits et cree les fichiers uty de characterisation correspondant
	 * 
	 * @param dirFits
	 *            Le repertoire dans lequel on va chercher les fichiers fits.
	 * @param mappingFile
	 *            fichier de mapping
	 * @param dirUtype
	 *            le repertoire qui va contenir les fichiers utypes cres
	 */
	public File createUtypeDirectory(File dirUtype, File dirFits,File mappingFile) {
		if (!dirFits.isDirectory()) {
			System.out.println(dirFits.getName() + "n'est pas un rpertoire");
			// autres traitements  ajouter
			// ecriture du log
		} else {
			// on recupere que les fichiers portant l'extension fits
			File[] fits = dirFits.listFiles(new Filtre(".fits"));

			if (!dirUtype.isDirectory() || !dirUtype.exists())
				dirUtype.mkdir();

			for (int i = 0; i < fits.length; i++) {
				String parfileName = fits[i].getName();
				parfileName = MappingTools.ChangeFileExtension(parfileName,
				"uty");
				File utypeF = new File(dirUtype.getAbsolutePath()
						+ File.separator + parfileName);
				this.createUtypeFile(utypeF, fits[i], mappingFile);

			}
		}
		return dirUtype;
	}

	/**
	 * cette fonction surcharge createUtypeDirectory(File dirUtype,File
	 * dirFits,File mappingFile ) les uty qui seront crees vont se mettre dans
	 * un rproire cree automatiquement, le repertoire sera cree au meme niveau
	 * que celui des fichiers Fits et aura comme nom le nom du repertoire des
	 * fichiers fits concatn avec Utypes exemple:
	 * fitsDirecory==>fitsDirecoryUtypes
	 * 
	 * @param dirFits
	 * @param mappingFile
	 * @return le repertoire des uty ainsi cree
	 */

	public File createUtypeDirectory(File dirFits, File mappingFile) {

		String dirFitsName = dirFits.getName();
		String dirFitsParentPath = dirFits.getParent();
		File dirUtype = new File(dirFitsParentPath + File.separator
				+ dirFitsName + "Utypes");
		this.createUtypeDirectory(dirUtype, dirFits, mappingFile);
		return dirUtype;
	}

	/**
	 * Cette fonction permet de gnrer la charaterisation XML d'un fichier Fits
	 * en utilisant un fichier de mapping
	 * 
	 * @param xmlF
	 *            le fichier XML  gnrer
	 * @param fitsF
	 *            le fichier FITS
	 * @param mapF
	 *            le fichier de Mapping
	 * @return le fichier charc XML ou null lorsque le fichier xmlF pass en
	 *         parametre n'existe pas
	 */

	public File generateXMLcharacFmFits(File xmlF, File fitsF, File mapF) {

		File utypeF = createUtypeFile(fitsF, mapF);
		

		/*File tempF = new File(xmlF.getParent());
		if (!tempF.exists()) {
			tempF.mkdir();// il faut traiter ici l'exception fileNoteFound
			System.out.println("xml parentFile not founded");
			return null;
		} else*/ {
			CharacterisationType ct = CharacConversions.fromUTypetoObject(utypeF);
			CharacConversions.fromObjectToXML(ct, xmlF);
		}
		return xmlF;

	}

	/**
	 * cette fonction surcharge generateXMLcharacFmFits(File xmlF,File
	 * fitsF,File mapF) le fichier Charc XML qui sera cr aura un nom gnr
	 * automatiquement  partir du fichier FITS exemple :
	 * nomFits.fits==>nomFits.xml le repertoire conteneur du fichier gnr est :
	 * dfltXMLFolder
	 * 
	 * @param fitsF
	 *            fichier fits
	 * @param mapF
	 *            fichier de mapping
	 * @return le fichier Charac. XML ainsi gnr
	 */
	public File generateXMLcharacFmFits(File fitsF, File mapF) {

		File xmlF = null;
		// construction du nom de fichier xml
		String xmlFileName = fitsF.getName();
		xmlFileName = MappingTools.ChangeFileExtension(xmlFileName, "xml");

		if (!(dfltXmlFolder.exists() && dfltXmlFolder.isDirectory())) {
			dfltXmlFolder.mkdir();
			System.out.println("Creation du rpertoire: ." + File.separator
					+ dfltXmlFolder.getName());
		}

		String rep = dfltXmlFolder.getName();
		xmlF = new File(rep + File.separator + xmlFileName);

		this.generateXMLcharacFmFits(xmlF, fitsF, mapF);

		return xmlF;

	}

	/**
	 * Cette fonction cherche dans un repertoire donn en parametre tous les
	 * fichiers fits et cree les fichiers Charac. XML correspondant
	 * 
	 * @param dirXml
	 *            le repertoire qui va contenir les fichier Charac gnrs
	 * @param dirFits
	 * @param mappingFile
	 * @return le repertoire des fichiers Charc. XML cre
	 */
	public File createXmlDirectoryFmFits(File dirXml, File dirFits,File mappingFile){
		File utypeDir = this.createUtypeDirectory(dirFits, mappingFile);

		// on recupere que les fichiers portant l'extension uty
		File[] utypes = utypeDir.listFiles(new Filtre(".uty"));
		if (!dirXml.isDirectory() || !dirXml.exists())
			dirXml.mkdir();

		for (int i = 0; i < utypes.length; i++) {
			String xmlFileName = utypes[i].getName();
			xmlFileName = MappingTools.ChangeFileExtension(xmlFileName, "xml");
			File xmlF = new File(dirXml.getAbsolutePath() + File.separator
					+ xmlFileName);

			CharacterisationType ct = CharacConversions
			.fromUTypetoObject(utypes[i]);
			CharacConversions.fromObjectToXML(ct, xmlF);

		}
		return dirXml;
	}

	/**
	 * cette fonction surcharge createXmlDirectoryFmFits(File dirXml,File
	 * dirFits,File mappingFile ) le repertoire des fichiers Charc XML qui sera
	 * cr aura un nom gnr automatiquement  partir du nom du repertoire des
	 * fichiers FITS exemple : nomDirFits==>nomDirFitsXml le repertoire anisi
	 * cre sera situ au meme niveau que le repertoire des fichier fits
	 * 
	 * @param dirFits
	 * @param mappingFile
	 * @return le repertoire des fichiers Charc. XML cre
	 */
	public File createXmlDirectoryFmFits(File dirFits, File mappingFile) {
		String dirFitsName = dirFits.getName();
		String dirFitsParentPath = dirFits.getParent();
		File dirXml = new File(dirFitsParentPath + File.separator + dirFitsName
				+ "Xml");
		return (this.createXmlDirectoryFmFits(dirXml, dirFits, mappingFile));

	}

	/**
	 * cette fonction permet de generer un fichier XML  apartir d'un fichier
	 * uty
	 * 
	 * @param xmlF
	 *            le fichier XML qui sera gnr
	 * @param utypeF
	 *            le fichier ytu
	 * @return le fichier XML gnr
	 */

	public File generateXMLcharacFmUtype(File xmlF, File utypeF) {

		/*File tempF = new File(xmlF.getParent());
		if (!tempF.exists()) {
			// tempF.mkdir();//il faut traiter ici l'exception fileNoteFound
			System.out.println("xml parentFile not founded");
		} else */{
			CharacterisationType ct = CharacConversions
			.fromUTypetoObject(utypeF);
			CharacConversions.fromObjectToXML(ct, xmlF);
		}
		return xmlF;

	}

	/**
	 * cette fonction surcharge la fonction generateXMLcharacFmUtype(File
	 * xmlF,File utypeF) le fichier Charac. XML ainsi gnr aura un nom gnrer
	 * automatiquement  partir du fichier uty exemple :
	 * nomFits.fits==>nomFits.xml
	 * le fichier XML sera cre dans le repertoire des fichier XML par defaut
	 * 
	 * @param utypeF
	 *            le fichier uty
	 * @return le fichier XML gnr
	 */

	public File generateXMLcharacFmUtype(File utypeF) {

		File xmlF = null;
		// construction du nom de fichier xml
		String xmlFileName = utypeF.getName();
		xmlFileName = MappingTools.ChangeFileExtension(xmlFileName, "xml");

		if (!(dfltXmlFolder.exists() && dfltXmlFolder.isDirectory())) {
			dfltXmlFolder.mkdir();
			System.out.println("Creation du rpertoire: ." + File.separator
					+ dfltXmlFolder.getName());
		}

		String rep = dfltXmlFolder.getName();
		xmlF = new File(rep + File.separator + xmlFileName);
		return (this.generateXMLcharacFmUtype(xmlF, utypeF));

	}

	/**
	 * Cette fonction cherche dans un repertoire donn en parametre tous les
	 * fichiers uty et cree les fichiers Charac. XML correspondant
	 * 
	 * @param dirXml
	 *            le repertoire qui va contenir les fichiers Charac. XML gnrs
	 * @param utypeDir
	 *            le repertoire des fichiers uty
	 * @return le repertoire des fichiers Charac. XML gnrs
	 */
	public File createXmlDirectoryFmUtypes(File dirXml, File utypeDir) {

		// on recupere que les fichiers portant l'extension uty
		File[] utypes = utypeDir.listFiles(new Filtre(".uty"));
		if (!dirXml.isDirectory() || !dirXml.exists())
			dirXml.mkdir();

		for (int i = 0; i < utypes.length; i++) {
			String xmlFileName = utypes[i].getName();
			xmlFileName = MappingTools.ChangeFileExtension(xmlFileName, "xml");
			File xmlF = new File(dirXml.getAbsolutePath() + File.separator
					+ xmlFileName);

			CharacterisationType ct = CharacConversions
			.fromUTypetoObject(utypes[i]);
			CharacConversions.fromObjectToXML(ct, xmlF);

		}
		return dirXml;
	}

	/**
	 * cette fonction surcharge createXmlDirectoryFmUtypes(File dirXml, File utypeDir)
	 * le repertoire des fichiers Charc XML qui sera
	 * cr aura un nom gnr automatiquement  partir du nom du repertoire des
	 * fichiers uty exemple : nomDirUtype==>nomDirUtypeXml le repertoire anisi
	 * cre sera situ au meme niveau que le repertoire des fichier uty
	 * 
	 * @param utypeDir le repertoire des fichier uty
	 * @return le repertoire des fichiers Charac. XML cre
	 */

	public File createXmlDirectoryFmUtypes(File utypeDir) {

		String dirXmlName = utypeDir.getName() + "Xml";
		String dirXmlParentPath = utypeDir.getParent();

		File dirXml = new File(dirXmlParentPath + File.separator + dirXmlName);

		return(this.createXmlDirectoryFmUtypes(dirXml, utypeDir));

	}
	/**
	 * Cette fonction retourne le repertoire des fichiers XML par defaut
	 * @return le repertoire des fichiers XML par defaut
	 */
	public File getDfltXmlFolder() {
		return dfltXmlFolder;
	}
	/**
	 * Cette fonction permet de modifier le repertoire des fichiers XML par defaut
	 * @param dfltXmlFolder
	 */

	public void setDfltXmlFolder(File dfltXmlFolder) {
		this.dfltXmlFolder = dfltXmlFolder;
	}
	/**
	 * Cette fonction retourne le repertoire des fichiers uty par defaut
	 * @return le repertoire des fichiers XML par defaut
	 */
	public File getDfltUtypeFolder() {
		return dfltUtypeFolder;
	}
	/**
	 * Cette fonction permet de modifier le repertoire des fichiers uty par defaut
	 * @param dfltUtypeFolder
	 */
	public void setDfltUtypeFolder(File dfltUtypeFolder) {
		this.dfltUtypeFolder = dfltUtypeFolder;
	}

	public static void main(String[] args) {
		
/*
		//fichier  fits 
		  File fits = new File("./2MASS/003.7858-39.2202.fits"); 
		  // fichiers de mapping
		  File mapsD = new  File("./mappingFiles/DenisSpe.map"); //
		  File maps2 = new File("./maapingFiles/2massAxesSpecifiques.map");
		  //fichier utype
		  File utype = new File("test1.uty"); //
		  //repertoire des fichiers fits
		  File dirFits=new File("./2MASS");
		  //repertoire des fichiers utypes
		  File dirUtype=new File("./dirParfiles");
		//repertoire des fichiers xml
		  File xmlF=new File("DirXml");
		  File xmlFile=new File("testXml.xml");
		  //objet CharacFile
		  CharacFile cf = new CharacFile();
		  //teste de la fonction createUtypeFile(utype,fits, maps2);
		  cf.createUtypeFile(utype,fits, maps2);
*/
		//teste de la fonction createUtypeFile(fits, maps2);
		// cf.createUtypeFile(fits, maps2);
		  
		//teste de la fonction createUtypeDirectory(File dirUtype, File dirFits,File mappingFile)
		  //cf.createUtypeDirectory(dirUtype,dirFits,maps2);
		  
		//teste de la fonction createUtypeDirectory(File dirFits,File mappingFile)
		 // cf.createUtypeDirectory(dirFits,maps2);
		  
		  //test de la fonction public File generateXMLcharacFmFits(File xmlF, File fitsF, File mapF)
		 //cf.generateXMLcharacFmFits(xmlFile,fits,maps2);
		 
		//test de la fonction public File generateXMLcharacFmFits(File fitsF, File mapF)
		 //cf.generateXMLcharacFmFits(fits,maps2);
		 
		//test de la fonction: File createXmlDirectoryFmFits(File dirXml, File dirFits,File mappingFile)
		 //cf.createXmlDirectoryFmFits(xmlF, dirFits,maps2);
		 
		//test de la fonction: File createXmlDirectoryFmFits(File dirFits,File mappingFile)
		// cf.createXmlDirectoryFmFits(dirFits,maps2);
		 
		//test de la fonction: File generateXMLcharacFmUtype(File xmlF, File utypeF)
		 //cf.generateXMLcharacFmUtype(xmlFile,utype);
		 
		//test de la fonction: File generateXMLcharacFmUtype( File utypeF)
		 //cf.generateXMLcharacFmUtype(utype);
		  
		//test de la fonction:File createXmlDirectoryFmUtypes(File dirXml, File utypeDir)
		 // cf.createXmlDirectoryFmUtypes(xmlF,dirUtype);
		  
		//test de la fonction:File createXmlDirectoryFmUtypes(File utypeDir)
		 // cf.createXmlDirectoryFmUtypes(dirUtype);
		  
		  
		 

	}

}
