package cds.charac.model;
import java.io.*;
import java.util.*;

import nom.tam.util.*;
import nom.tam.fits.*;


public class MoCharacGenerator {
	
	private File fitsFile;
	private File mapFile;
	private String sep="+";
	//cette variable va nous servir pour savoir si l'objet creer par le constructeur
	// a bien t cre
	boolean objectValide=true;
	//Pour detrminer le format de fichier  produire
	//  liminer, plus besoin de cette var.
	int param=1;
	//contient KeyWordFits,utypes :on la remplie  partir  du fichier de mapping
	//au niveau du constructeur
	private HashMap<String,String> mappingTable=new HashMap<String,String>();	
			
	/**
	 * Constructeur permettant de creer un objet de type MoCharacGenerator, lors de la creation de l'objet
	 * on la variable de classe  mappingTable sera remplie  l'aide de la fonction
	 * readMappingFile, si la creeation de l'objet releve une exception: la variable de classe
	 * objectValide prendra la valeur false
	 * @param fitsFile : Le fichier FITS
	 * @param mapFile  : Le fichier de Mapping
	 */
	public  MoCharacGenerator(File fitsFile, File mapFile) {
		super();
		try{
		this.fitsFile = fitsFile;
		this.mapFile = mapFile;
		this.readMappingFile();
		}catch( FileNotFoundException e){
			
			System.out.println("File not found :"+e.getMessage()) ;
			
			//ecriture du log
			this.objectValide=false;
			
		}catch( IOException e){
			
			//System.out.println(e.getMessage()) ;
			//ecriture du log
			this.objectValide=false;
			
		}	
	}
	//A re faire :
	//il faut enlever  le systeme.out et le remplacer par une valeur de retour de type false et
	//eventuelement ecriture du log

	/**
	 * cette fonction permet de lire le fichier de mapping et de remplir la variable de 
	 * classe mappingTable
	 * @exception FileNotFoundException
	 * @exception IOException
	 */
	public void readMappingFile()throws FileNotFoundException,IOException{

		FileReader fr= new FileReader(this.mapFile);
		BufferedReader br =new BufferedReader(fr);

		String line;
		while((line=br.readLine())!=null){
			//chaque ligne du fichier sera decompose selon le token +
			StringTokenizer st=new StringTokenizer(line,this.sep);	

			while(st.hasMoreTokens()){		

				this.mappingTable.put(st.nextToken().trim(),st.nextToken().trim());

			}
		}
		System.out.println("Reading Mapping File : "+this.mapFile.getParent()+File.separator+this.mapFile.getName()+"  effectue");
	}
	/**
	 * Cette fonction permet une recherche par keyword dans le fichier de mapping
	 * @param key : le keyWord  chercher
	 * @return true si le keyword exisite , false sinon
	 */
	public boolean searchInMapF(String key){
		return this.mappingTable.containsKey(key.trim());
	}
	
	/**
	 * cette fonction permet de retourner les keyWds communs entre le fichier de mapping et le fichier FITS
	 * @return un vecteur des keywds communs entre le fichier fits et celui de mapping,
	 * les elements du vecteur ont le meme ordre que celui du fichier fits
	 * 
	 */
	public Vector<String>CommonKeys() {		

		Vector<String> retour=new Vector<String>();
		if(this.objectValide)
		{
			try{
				Fits fi=new Fits(this.fitsFile);
				ImageHDU ih=(ImageHDU)fi.readHDU();

				if(fi.getNumberOfHDUs()!=0){				
					Header hdr=ih.getHeader();
					Cursor iter =hdr.iterator();

					while (iter.hasNext()) {
						HeaderCard hc = (HeaderCard) iter.next();
						String keyFits=hc.getKey();

						if(this.searchInMapF(keyFits.trim()))
						{
							retour.addElement(keyFits);

						}				      
					}
				}
			}catch(FitsException e){
				System.out.println("Fits Error: "+e.getMessage());
				this.objectValide=false;
				//WinGenFmFits.erreur(e.getMessage());
			}
			catch(IOException e){
				this.objectValide=false;
				System.out.println("une error d'E/S est survenue");
				//WinGenFmFits.erreur(e.getMessage());

			} 
		}
		return retour;

	}
	/**
	 * Cette fonction permet de retourner la valeur d'un keyword donn en parametre
	 * 
	 * @param keywd le keyword
	 * @return la valeur du keyword s'il existe, null sinon
	 */
	
	public String valueOfKeyWord(String keywd){
		String retour="Incorrect";
		if(this.objectValide)
		{
			try{
				Fits fi=new Fits(this.fitsFile);
				ImageHDU ih=(ImageHDU)fi.readHDU();
				if(fi.getNumberOfHDUs()!=0){				
					Header hdr=ih.getHeader();
					Cursor iter =hdr.iterator();
					while (iter.hasNext()) {
						HeaderCard hc = (HeaderCard) iter.next();
						if(keywd.trim().compareTo(hc.getKey())==0)
						{
							if(hc.getValue()!=null)
							{
								retour=interpretFitsValues(hc.getValue().trim());
								//System.out.println("1\n"+hc.toString());
								
							}
							else{//probleme au niveau de la librairie fits qui n'arrive pas
								// a extraire la valeur du keyword
								//Solution
								//on extrait le value du keyword,  l'aide des fonctions
								//toString() et split()
								String[] val=hc.toString().split("'");
								// on formate la valeur du keyWord pour assurrer la compatibilit 
								//avec la libraire:characterisationLibrary
								retour=interpretFitsValues(val[1].trim());

							}
							break;
						}
					}	
				}
			}catch(FitsException e){//seule cette exception qui doit etre traite soignesement
				//les autres sont traites au niveau de la creation de l'objet et ne peuvent
				//jamais survenir
				this.objectValide=false;
				System.out.print("Fits Error: "+e.getMessage());
				//WinGenFmFits.erreur("Fits Error: "+e.getMessage());
				//ecriture du log a prevoir			
			}
			catch(IOException e){
				System.out.println("une error d'E/S est survenue");
				this.objectValide=false;

			}catch (NullPointerException e){
				this.objectValide=false;
				System.out.println(e.getMessage());			
			}
		}
		return retour;
	}
	/**
	 * cette fonction retourne un vector contenant les indices des des KeyWords correspondant aux axisName des axes generiques,
	 * la rechere des keyWord se fait dans le vecteur retourn par la fonction CommonKeys()
	 * @return vecteur des indices des AxisName
	 */	
	
	public Vector<Integer> indexOfGenericAxisName(){		
		
		Vector<Integer> vAxisIndex=new Vector<Integer>();
		if(this.objectValide)
		{
		//les keywords communs entre mappingtable et fitsFile
		Vector<String> vComm=CommonKeys();		
		//recherche des indices des KeyWords correspondant aux axisName
		String temp;		
		for(int i=0;i<vComm.size();i++){
			{
				temp=vComm.elementAt(i).trim();
				//on recupere le utype correspondant
				temp=this.mappingTable.get(temp);
				if((temp.contains("CharacterizationAxis")||temp.contains("CharacterisationAxis"))&&temp.contains(".AxisName"))
						vAxisIndex.add(i);
			}					
		}
		
		//System.out.println("Nombre d'axes generiques "+vAxisIndex.size());
		
		}
		return vAxisIndex;
	}
	/**
	 * Cette fonction permet de retourner un vecteur des hashMap,dont chacun contient
	 * la charaterization (utype, value) d'un axe generique
	 * @return vecteur des HashMap
	 */
		
	public  Vector<HashMap<String, String>> genericAxis(){
		Vector<HashMap<String, String>> hm=new Vector<HashMap<String, String>>();
		if(this.objectValide)
		{
		Vector<Integer> vAxis=this.indexOfGenericAxisName();
		Vector<String> vComm=this.CommonKeys();
		
		if(vAxis.size()!=0){
			HashMap<String,String> shm;
			int borne=0;
			
			for(int i=0;i<vAxis.size();i++){
				{
					if(i==vAxis.size()-1)
						borne=vComm.size();
					else
						borne=vAxis.elementAt(i+1);
						shm=new HashMap<String,String>();
						for(int j=vAxis.elementAt(i);j<borne;j++){
							String temp=this.mappingTable.get(vComm.elementAt(j));
							//on fait ce test au cas ou le fichier fits contient  la fois des axes specifiques et generiques
							//dans ce cas si les keyWdFits d'un axes specifique sont situs au dessous de ceux d'un axe generiques
							//alors ils vont s'ajouter au ce dernier, ce qui va fausser les resultats attendus   
							//d'ou la necessit du test
							if((temp.contains("CharacterizationAxis")||temp.contains("CharacterisationAxis")))
								//ce teste pour ignorer les utypes ayant la  valeur non significatif unknown
								if(this.valueOfKeyWord(vComm.elementAt(j)).trim().toLowerCase().equals("unknown"))
									continue;
								shm.put(this.mappingTable.get(vComm.elementAt(j)),this.valueOfKeyWord(vComm.elementAt(j)));
							//System.out.println(j+1+" "+this.mappingTable.get(vComm.elementAt(j))+" "+this.valueOfKeyWord(vComm.elementAt(j)));
						
					}
					hm.add(shm);				
				}
			}
				
		}
		
		}
		return hm;
	}
	/**
	 * Cette fonction permet de rcuperer la liste d'axisName specifiques communs entre
	 * le fichier Fits et le fichier de mapping, valable uniquement pour les axes specifiques
	 * @return un vecteur contenant le nom des axes spcifiques communs entre le fichier Fits et le fichier de mapping 
	 */		
	private Vector<String> listOfSpecificAxisName(){			

		Vector<String> vectAxis=new Vector<String>();
		if(this.objectValide)
		{		
			Vector<String> vComm=this.CommonKeys();
			for(int i=0;i<vComm.size();i++){
				if(this.searchInMapF(vComm.elementAt(i))){
					String utype=this.mappingTable.get(vComm.elementAt(i));
					if(!utype.contains("CharacterizationAxis")&& !utype.contains("CharacterisationAxis"))
					{	
						String tempAxisName[]=utype.split("Axis.");
						tempAxisName[0]=tempAxisName[0].trim();
						if(!vectAxis.contains(tempAxisName[0]))
						{
							vectAxis.addElement(tempAxisName[0]);

						}
					}
				}
			}
		}

		return vectAxis;

	}
	/**
	 * Cette fonction permet de retourner un vecteur des hashMap,dont chacun contient
	 * la charaterization (utype,value) d'un axe generique
	 * @return vecteur des HashMap
	 */	
	public  Vector<HashMap<String, String>> specificAxis(){
		Vector<HashMap<String, String>> hm=new Vector<HashMap<String, String>>();
		if(this.objectValide)
		{
		Vector<String> vAxisName=this.listOfSpecificAxisName();
		Vector<String> vComm=this.CommonKeys();
		HashMap<String,String> shm;
		for(int i=0;i<vAxisName.size();i++){
			shm=new HashMap<String, String>();
			String tmpAxisName=vAxisName.elementAt(i).trim();
			for(int j=0;j<vComm.size();j++){
				String key=vComm.elementAt(j);
				if(this.mappingTable.containsKey(key)){
					String ut=this.mappingTable.get(key);
					if(ut.trim().startsWith(tmpAxisName)){
						//ce teste pour ignorer les utypes ayant la  valeur non significatif unknown
						if(this.valueOfKeyWord(key).trim().toLowerCase().equals("unknown"))
							continue;
						shm.put(ut,this.valueOfKeyWord(key));
					}
				}
			}		
			hm.add(shm);
		}
		
		}
		return hm;
	}	
	/**
	 * Cette fonction permet de formater la valeur de la chaine de caractere donne
	 * en parametre representant des valeurs a plusiurs dimensions qu'on extrait du 
	 * headerFits de l'image,selon le format suport par la libraire characterisationLibrary
	 * @param string
	 * @return la chaine formate
	 */
	public static String interpretFitsValues(String string){
		string=string.trim();
		String retour=string;
		String[] val=string.split("\\s");
		//System.out.println("s: "+val.length+"   "+val[0]);

		if(val.length>=2 ){
			retour="(";
			for(int i=0;i<val.length;i++ ){
				if(val[i].trim().length()==0)
					continue;
				retour=retour+val[i];
				if(i<val.length-1)
					retour=retour+",";
			}
			retour=retour+")";
		}
		else//string ne contient pas \s
		{
			val =string.split("-");
			//System.out.println("\n-: "+val.length +" "+val[1]);
			//if(val[0].trim()=="")
			if(val.length==2 && (val[0].trim().length()!=0)){
				retour="(";
				for(int i=0;i<val.length;i++ ){
					if(val[i].trim().length()==0)
						continue;
					if(i==0)
						retour=retour+val[i];
					else
						retour=retour+"-"+val[i];
					if(i<val.length-1)
						retour=retour+",";
				}
				retour=retour+")";
			}
			else{
				val=string.split("\\+");
			
				if(val.length>=2 && (val[0].trim().length()!=0)){
					retour="(";
					for(int i=0;i<val.length;i++ ){
						if(val[i].trim().length()==0)
							continue;
						retour=retour+val[i];
						if(i<val.length-1)
							retour=retour+",";
					}
					retour=retour+")";
				}

			}

		}

		return retour;
	}
	
	/**
	 * fonction provisoire pour lire un fichier fits et reperer les valeurs des keyWd fits qui ont la valeurs
	 * null
	 * @param keywd
	 * @return
	 */
	public static Vector<String> readFits(File fitsFile){
		Vector<String> retour=new Vector<String>();
		try{
			Fits fi=new Fits(fitsFile);
			ImageHDU ih=(ImageHDU)fi.readHDU();
		
			if(fi.getNumberOfHDUs()!=0){
				
				Header hdr=ih.getHeader();
				Cursor iter =hdr.iterator();
				
				int i=0;
				while (iter.hasNext()) {
					HeaderCard hc = (HeaderCard) iter.next();					
					System.out.println(i+ "  keyWord="+hc.getKey()+ "  value=  "+ hc.getValue()+"  Comment="+hc.getComment());
					System.out.println(hc.toString());
					if(hc.getValue()==null)
						retour.add(hc.toString());
					i++;
				}	
			}
		}catch(FitsException e){			
			System.out.print("Fits Error: "+e.getMessage());
		}
		catch(IOException e){
			System.out.println("une error d'E/S est survenue");
		}catch (NullPointerException e){
			System.out.println(e.getMessage());
		}
		return retour;
	}
	
	public static  String[] fitsKeyWords(File fits){
		String [] KeyWords=null;
		try{
			Fits fi = new Fits(fits);
			ImageHDU h= (ImageHDU)fi.readHDU();
			Header hdr = h.getHeader();		
			Cursor iter = hdr.iterator();
			KeyWords=new String[hdr.getNumberOfCards()]; 
			int i=0;
			while (iter.hasNext()) {
				HeaderCard hc = (HeaderCard) iter.next();
				//KeyWords[i]=hc.getKey();
				KeyWords[i]=hc.toString();
				i++;
			}
		}catch(FitsException e){
			System.out.print("Error lors du chargement du fichier:"+fits.getName());
			System.exit(0);
		}
		catch(IOException io){System.out.print("Probleme d'I/O");}
		return KeyWords;		
	}	
	public static void main(String[] args){

		//File fits = new File("./011.7866-20.7605.fits");
		// fichier de mapping
		//File mapping2spe = new File("./mappingFiles/MapAxeGenerique2.map");
		//File mapping2sp = new File("./mappingFiles/MappingAxesSpecifiquesL.map");
		//MoCharacGenerator mo= new MoCharacGenerator(fits,mapping2sp);
		//mo.CommonKeys();
		//mo.indexOfGenericAxisName();
		//System.out.println(mo.valueOfKeyWord("MAXWAVEL"));
		/*mo.test();
		
		System.out.println(mo.genericAxis().toString());
		mo= new MoCharacGenerator(fits,mapping2sp);
		System.out.println("le progr n'est pas arret"+" et valeur de Object Valide ="+mo.objectValide);
		System.out.println("Axis Generiqiques Characterisation\n:"+mo.genericAxis().toString());
		System.out.println("Axis Specifiques:"+mo.listOfSpecificAxisName());
		System.out.println("Axis Specifiques Characterisation :\n"+mo.specificAxis().toString());
		*/
		/*
		String s=interpretFitsValueNull("+0.000277777784317036+0.000277777784317036+oiuiuiu");
		System.out.println(s);*/
		//String s=" 0.000277777784317036-0.000277777784317036-";
		//String sep="-";
		//spliter(s,sep);
	    //String[] s=fitsKeyWords(fits);
	    /*for(int i=0;i<s.length;i++)
	    	System.out.println(s[i]);*/
	    
	    
	    /*
		Vector<String> nullValue=readFits(fits);
		System.out.println("\n\n les valeur de null:\n");
		for(int i=0;i<nullValue.size();i++)
			System.out.println(i+"	"+nullValue.elementAt(i));*/
		
	}
	
}
